/// <reference types="mongoose" />
import 'reflect-metadata';
import { Document, Model as MModel, Mongoose, Schema, SchemaOptions } from 'mongoose';
import { MongoError } from 'mongodb';
export interface Model extends Document {
}
export declare type ModelType = typeof Model;
export declare type IModel<E extends Model> = MModel<E>;
/**
 * Wrapped Model from mongoose.Model.
 */
export declare class Model {
    static $CONFIG: SchemaOptions;
    static $SCHEMA: {};
    private static $PRES;
    private static $POSTS;
    private static $VIRTUALS;
    private static $PLUGINS;
    private static $INDEXES;
    private static $MIXINS;
    private static $VALIDATES;
    private static _mongooseOptions;
    static Config(config: SchemaOptions): ModelType;
    static Schema(schema: {}): ModelType;
    static Pre(pre: Pre): ModelType;
    static Post(post: Post): ModelType;
    static Virtual(virtual: Virtual): ModelType;
    static Plugin(plugin: Plugin): ModelType;
    static Index(index: Index): ModelType;
    static Mixin(model: ModelType): ModelType;
    static Validate(validate: Validate): ModelType;
    private static _$modifySchema(discriminatorKey);
    static $mongooseOptions(): MongooseOptions;
    static $register<D extends Document, M>(mongooseInstance?: Mongoose): MModel<D> & M;
    static _$initialize(): void;
    static cast<D extends Model>(): IModel<D>;
}
export declare function Field(schema: any): (target: any, propertyName: string) => void;
/**
 * Mongoose options for current model.
 */
export interface MongooseOptions {
    initialized: boolean;
    config?: SchemaOptions;
    schema?: {};
    mongooseSchema?: Schema;
    pres?: Pre[];
    posts?: Post[];
    virtuals?: Virtual[];
    methods?: Method[];
    statics?: Method[];
    plugins?: Plugin[];
    indexes?: Index[];
    validates?: Validate[];
}
export interface Pre {
    name: string;
    fn: (next: (err?: NativeError) => void) => void;
    parallel?: boolean;
    errorCb?: (err: Error) => void;
}
export interface Post {
    name: string;
    fn: PostFn1 | PostFn2;
}
export declare type PostFn1 = (doc: object, next: (err?: NativeError) => void) => void;
export declare type PostFn2 = (error: MongoError, doc: object, next: (err?: NativeError) => void) => void;
export interface Virtual {
    name: string;
    get?: () => any;
    set?: (val?: any) => void;
}
export interface Method {
    name: string;
    fn: () => void;
}
export interface Plugin {
    fn: (schema: Schema, options?: object) => void;
    options?: object;
    priority?: number;
}
export interface Index {
    fields: object;
    options?: {
        expires?: string;
        [other: string]: any;
    };
}
export interface Validate {
    path: string;
    fn: (val?: any) => boolean;
    errorMsg?: string;
    type?: string;
}
export declare class NativeError extends global.Error {
}
export declare const preQueries: string[];
